/******************************************************************************
*                                                                             *
*                            Module specification                             *
*                                                                             *
*                                Copyright by                                 *
*                                                                             *
*                              Azoteq (Pty) Ltd                               *
*                          Republic of South Africa                           *
*                                                                             *
*                           Tel: +27(0)21 863 0033                            *
*                          E-mail: info@azoteq.com                            *
*                                                                             *
*******************************************************************************
Name             :  lower level PIC18F4550.c
Description      :  PIC specific functions for IQS316 SPI Firmware library
*******************************************************************************/

#include "includes.h"

//*****************************************************************************
//
//! Initialise
//!
//! Initializes the PIC18F4550
//!
//! \param None
//!
//! \return None
//
//*****************************************************************************
void
init(void)
{
    LATB = 0x00;                //PORTB all outputs - low
    TRISB = 0x00;               //IQS316 MCLR on B2 - hold in reset

    TRISD = 0x00;               //configure PORTD for output
    LATD = 0x0F;                //LEDS off

    LATC = 0x00;                //start by setting debug io
    TRISC = 0xF8;               //RC0-RC2 used for debug io's

    ADCON1 = 0x0F;              //PORTA all digital operation

    LATA = 0x00;
    TRISA = 0x00;               //All outputs

    INTCON2bits.RBPU = 0;       //For PIC18LF4550-Eval-Rev4A hardware with 74HC573 latch
    TRISCbits.TRISC0 = 0;
    LATCbits.LATC0  = 1;        //un-latch 74HC573 to make OUTD follows LATD
    TRISCbits.TRISC1 = 0;       //enable the latch
    LATCbits.LATC1 = 0;
    //
    // Delay 1ms to allow MCLR to go low (since there is normally a 100nF
    // capacitor on MCLR.  This gives the IQS316 a nice clean reset.
    //
    DelayMS(1);

    Comms_init();

    //
    // Add delay here to allow IQS316 to startup
    // According to datasheet first comms is available roughly 16ms
    // after MCLR is released
    //
    DelayMS(15);

    // Place other functions responsible for hardware initialization here.

    IQS316_Settings();
}

//*****************************************************************************
//
//! SPI Initialise
//!
//! Initializes the SPI module on the PIC18F4550
//!
//! \param None
//!
//! \return None
//
//*****************************************************************************
void
Comms_init()
{
    TRISB |= 0x01;                  //SOMI input on B0
    TRISB &= 0xFD;                  //SCK output on B1
    TRISA |= 0x01;                  //RDY input on RA0
    TRISA &= 0xFD;                  //SS output on RA1
    TRISC &= 0x7F;                  //MOSI output on RC7

    SSPSTAT = 0x80;
    SSPCON1 = 0x32;                 //enables SSP, SCK idle high

    LATA |= 0x02;                   //Set SS high

    LATB = LATB | 0x04;             //IQS316 MCLR High
}


//*****************************************************************************
//
//! Perform an SPI comms transfer
//!
//! The function firstly waits until RDY indicates that a communication window
//! is active.  The IQS316 slave device is then selected by pulling SS low.
//! The data transfer is then performed, with a byte transmitted on MOSI and
//! received simultaneously on SOMI.  SS is released again after the
//! transmission is complete.
//!
//! \param ui8SendData is the data to transmit on the MOSI pin
//!
//! \return Received byte from SOMI is returned from the function
//
//*****************************************************************************
uint8_t
CommsIQS316_RxTx(uint8_t ui8SendData)
{
    uint8_t ui8ReceiveData;
    //
    // Wait for ready signal
    //
    while(RDY == 0)
    {}
    //
    // Select IQS316 by pulling SS low
    //
    LATA = LATA & 0xFD;
    //
    // reset transmission complete flag
    //
    PIR1bits.SSPIF = 0;
    //
    // Perform read
    //
    ui8ReceiveData = SSPBUF;
    //
    // Initiate transmission
    //
    SSPBUF = ui8SendData;
    //
    // Wait for transmission complete flag
    //
    while (PIR1bits.SSPIF == 0)
    {}
    //
    // Temp store received byte
    //
    ui8ReceiveData = SSPBUF;
    //
    // Release SS line on IQS316
    //
    LATA = LATA | 0x02;
    
    return ui8ReceiveData;
}

// =============================================================================
// =========== PURELY FOR DEBUGGING - not part of library ======================

void
DelayMS(uint8_t ui8Milliseconds)
{
    uint16_t ui16i, ui16Count;
    uint16_t ui16LoopLength;

    ui16LoopLength = 500*(uint16_t)ui8Milliseconds;

    for(ui16i = 0 ; ui16i < ui16LoopLength ; ui16i++)
    {
        ui16Count++ ;
    }
}

void
DebugLED(uint8_t ui8LEDState, uint8_t ui8LEDNumber)
{
   if(ui8LEDState == CLEAR_LED)
   {
      // LED is inactive on an I/O HIGH state (sink)
      //
      LATD |= ui8LEDNumber;
   }
   else
   {
       // LED is active on an I/O LOW state (sink)
      LATD &= (uint8_t)~ui8LEDNumber;
   }
}

void
DebugIO(uint8_t ui8IOState, uint8_t ui8IONumber)
{
   if(ui8IOState == SET_IO)
   {
      LATC |= ui8IONumber;
   }
   else
   {
      LATC &= (uint8_t)~ui8IONumber;
   }
}

void
PulseIO(uint8_t ui8IONumber, uint8_t ui8NoOfPulses)
{
    uint8_t ui8i;

    for(ui8i = 0 ; ui8i < ui8NoOfPulses ; ui8i++)
    {
        LATD |= ui8IONumber;
        LATD &= (uint8_t)~ui8IONumber;
    }
}